@@ -30,7 +30,8 @@
 #include <QPainter>
 #include <QTimer>
 
-#define USE_UPDATE 1
+#include <phonon/audiooutput.h>
+
 
 #define EMPTY           0
 #define SEASONS_START   28
@@ -40,6 +41,7 @@ static int s_delay[5] = {1000, 750, 500, 250, 125};
 
 bool PossibleMove::isInPath(int x, int y) const
 {
+    kDebug() << "Entry";
     if (x == m_path.last().x && y == m_path.last().y) {
         return false;
     }
@@ -69,12 +71,13 @@ Board::Board(QWidget *parent)
     m_markX(0), m_markY(0),
     m_field(0),
     m_xTiles(0), m_yTiles(0),
-    m_delay(125), m_shuffle(0),
+    m_delay(0), m_shuffle(0),
     m_isPaused(false), m_isStuck(false), m_isOver(false), m_cheat(false),
     m_gravityFlag(true), m_solvableFlag(true), m_chineseStyleFlag(false), m_tilesCanSlideFlag(false),
     m_highlightedTile(-1), m_connectionTimeout(0),
-    m_paintConnection(false), m_paintPossibleMoves(false)
+    m_paintConnection(false), m_paintPossibleMoves(false), m_paintInProgress(false), m_media(0)
 {
+    kDebug() << "Entry";
     m_tileRemove1.first = -1;
 
     m_random.setSeed(0);
@@ -89,11 +92,13 @@ Board::Board(QWidget *parent)
 
 Board::~Board()
 {
+    kDebug() << "Entry";
     delete [] m_field;
 }
 
 void Board::loadSettings()
 {
+    kDebug() << "Entry";
     if (!loadTileset(Prefs::tileSet())) {
         qDebug() << "An error occurred when loading the tileset" << Prefs::tileSet() << "KShisen will continue with the default tileset.";
     }
@@ -109,7 +114,7 @@ void Board::loadSettings()
     // face because they are technically different (e.g different flowers).
     // The solution is to enforce chineseStyle gameplay for tile sets that are
     // known to be reduced. Those are Egypt and Alphabet for now.
-    if (Prefs::tileSet().endsWith("egypt.desktop") || Prefs::tileSet().endsWith("alphabet.desktop")) {
+    if (Prefs::tileSet().endsWith(QLatin1String("egypt.desktop")) || Prefs::tileSet().endsWith(QLatin1String("alphabet.desktop"))) {
         setChineseStyleFlag(true);
     } else {
         setChineseStyleFlag(Prefs::chineseStyle());
@@ -122,10 +127,12 @@ void Board::loadSettings()
     setSize(sizeX[Prefs::size()], sizeY[Prefs::size()]);
     setGravityFlag(Prefs::gravity());
     setDelay(s_delay[Prefs::speed()]);
+    setSoundsEnabled(Prefs::sounds());
 }
 
 bool Board::loadTileset(const QString &pathToTileset)
 {
+    kDebug() << "Entry";
     if (m_tiles.loadTileset(pathToTileset)) {
         if (m_tiles.loadGraphics()) {
             Prefs::setTileSet(pathToTileset);
@@ -147,11 +154,13 @@ bool Board::loadTileset(const QString &pathToTileset)
 
 bool Board::loadBackground(const QString &pathToBackground)
 {
+    kDebug() << "Entry";
     if (m_background.load(pathToBackground, width(), height())) {
         if (m_background.loadGraphics()) {
             Prefs::setBackground(pathToBackground);
             Prefs::self()->writeConfig();
             resizeBoard();
+            kDebug() << "Exit on success";
             return true;
         }
     }
@@ -163,6 +172,7 @@ bool Board::loadBackground(const QString &pathToBackground)
             resizeBoard();
         }
     }
+    kDebug() << "Exit on failure (fallback to Default)";
     return false;
 }
 
@@ -186,6 +196,7 @@ void Board::setField(int x, int y, int value)
     m_field[y * xTiles() + x] = value;
 }
 
+// TODO: Why is this called every second?
 int Board::field(int x, int y) const
 {
 #ifdef DEBUGGING
@@ -202,28 +213,28 @@ int Board::field(int x, int y) const
     return m_field[y * xTiles() + x];
 }
 
-/**
- * @param update FIXME: What is it for?
- */
 void Board::gravity(bool update)
 {
+    kDebug() << "Entry";
     m_gravCols.clear();
     if (!m_gravityFlag) {
         return;
     }
+    bool fallingTiles = false;
     for (int i = 0; i < xTiles(); ++i) {
         if (gravity(i, update)) {
+            fallingTiles = true;
             m_gravCols.append(i);
         }
     }
+    if (Prefs::sounds() && fallingTiles) {
+        playSound(KStandardDirs::locate("sound", "kshisen/tile-fall-tile.ogg"));
+    }
 }
 
-/**
- * @param column The column to check
- * @param update FIXME: What is it for?
- */
 bool Board::gravity(int column, bool update)
 {
+    kDebug() << "Entry";
     bool affected = false;
     if (m_gravityFlag) {
         int rptr = yTiles() - 1;
@@ -254,6 +265,15 @@ bool Board::gravity(int column, bool update)
 
 void Board::mousePressEvent(QMouseEvent *e)
 {
+    kDebug() << "Entry";
+    // Do not process mouse events while the connection is drawn.
+    // Clicking on one of the already connected tiles would have selected
+    // it before removing it. This is more a workaround than a proper fix
+    // but I have to understand the usage of m_paintConnection first in
+    // order to consider its reusage here. (schwarzer)
+    if (m_paintInProgress) {
+        return;
+    }
     if (m_isOver) {
         newGame();
         return;
@@ -331,10 +351,6 @@ void Board::mousePressEvent(QMouseEvent *e)
     }
 }
 
-/**
- * The board is centred inside the main playing area. xOffset/yOffset provide
- * the coordinates of the top-left corner of the board.
- */
 int Board::xOffset() const
 {
     int tw = m_tiles.qWidth() * 2;
@@ -349,6 +365,7 @@ int Board::yOffset() const
 
 void Board::setSize(int x, int y)
 {
+    kDebug() << "Entry";
     if (x == xTiles() && y == yTiles()) {
         return;
     }
@@ -382,6 +399,7 @@ void Board::setSize(int x, int y)
 
 void Board::resizeEvent(QResizeEvent *e)
 {
+    kDebug() << "Entry";
     kDebug() << "[resizeEvent]";
     if (e->spontaneous()) {
         kDebug() << "[resizeEvent] spontaneous";
@@ -392,6 +410,7 @@ void Board::resizeEvent(QResizeEvent *e)
 
 void Board::resizeBoard()
 {
+    kDebug() << "Entry";
     // calculate tile size required to fit all tiles in the window
     QSize newsize = m_tiles.preferredTileSize(QSize(width(), height()), xTiles(), yTiles());
     m_tiles.reloadTileset(newsize);
@@ -401,11 +420,13 @@ void Board::resizeBoard()
     QPalette palette;
     palette.setBrush(backgroundRole(), m_background.getBackground());
     setPalette(palette);
+    kDebug() << "Exit";
 }
 
 
 void Board::newGame()
 {
+    kDebug() << "Entry";
     kDebug() << "NewGame";
 
     m_isOver = false;
@@ -425,6 +446,11 @@ void Board::newGame()
     // distribute all tiles on board
     int curTile = 1;
     int tileCount = 0;
+    if (m_chineseStyleFlag) {
+        kDebug() << "==== Chinese flag is set";
+    } else {
+        kDebug() << "==== Chinese flag is not set";
+    }
     /*
      * Note by jwickers: i changed the way to distribute tiles
      *  in chinese mahjongg there are 4 tiles of each
@@ -456,17 +482,21 @@ void Board::newGame()
         update();
         resetTimer();
         emit changed();
+        kDebug() << "Exit without shuffling";
         return;
     }
 
     // shuffle the field
     int tx = xTiles();
     int ty = yTiles();
-    for (int i = 0; i < xTiles() * yTiles() * shuffle(); ++i) {
+    for (int i = 0; i < tx * ty * shuffle(); ++i) {
         int x1 = m_random.getLong(tx);
         int y1 = m_random.getLong(ty);
         int x2 = m_random.getLong(tx);
         int y2 = m_random.getLong(ty);
+        // keep this, because the next setField() call changes what field() will return
+        // so there would a significant impact on shuffling with the field() call put into the
+        // place where 't' is used
         int t  = field(x1, y1);
         setField(x1, y1, field(x2, y2));
         setField(x2, y2, t);
@@ -477,6 +507,7 @@ void Board::newGame()
         update();
         resetTimer();
         emit changed();
+        kDebug() << "Exit if game does not need to be solvable";
         return;
     }
 
@@ -542,11 +573,9 @@ void Board::newGame()
     update();
     resetTimer();
     emit changed();
+    kDebug() << "Exit";
 }
 
-/**
- * This is sed for connecting them and for highlighting tiles of the same group.
- */
 bool Board::tilesMatch(int tile1, int tile2) const
 {
     // identical tiles always match
@@ -597,20 +626,18 @@ bool Board::isTileHighlighted(int x, int y) const
 
 void Board::updateField(int x, int y)
 {
+    kDebug() << "Entry";
     QRect r(xOffset() + x * m_tiles.qWidth() * 2,
             yOffset() + y * m_tiles.qHeight() * 2,
             m_tiles.width(),
             m_tiles.height());
 
-#ifdef USE_UPDATE
     update(r);
-#else
-    repaint(r);
-#endif
 }
 
 void Board::paintEvent(QPaintEvent *e)
 {
+    kDebug() << "Entry";
     QRect ur = e->rect();            // rectangle to update
     QPixmap pm(ur.size());           // Pixmap for double-buffering
     pm.fill(this, ur.topLeft());     // fill with widget background
@@ -627,6 +654,7 @@ void Board::paintEvent(QPaintEvent *e)
         p.setFont(font);
         //     p.setPen(QPen(QColor("black")));
         p.setBrush(QBrush(QColor(100, 100, 100, 150)));
+        p.setRenderHint( QPainter::Antialiasing );
         p.drawRoundedRect(contentsRect, 10, 10);
 
         if (m_isPaused) {
@@ -686,7 +714,7 @@ void Board::paintEvent(QPaintEvent *e)
     if (m_paintPossibleMoves) {
         p.setPen(QPen(QColor("blue"), lineWidth()));
         // paint all possible moves
-        for (QList<PossibleMove>::iterator iter = m_possibleMoves.begin(); iter != m_possibleMoves.end(); ++iter) {
+        for (QList<PossibleMove>::const_iterator iter = m_possibleMoves.constBegin(); iter != m_possibleMoves.constEnd(); ++iter) {
             // Path.size() will always be >= 2
             Path::const_iterator pathEnd = iter->m_path.constEnd();
             Path::const_iterator pt1 = iter->m_path.constBegin();
@@ -705,6 +733,7 @@ void Board::paintEvent(QPaintEvent *e)
 
 void Board::reverseSlide(int x, int y, int slideX1, int slideY1, int slideX2, int slideY2)
 {
+    kDebug() << "Entry";
     //kDebug() << "reverseSlide" << x << " " << y;
 
     // s_x2 is the current location of the last tile to slide
@@ -770,6 +799,7 @@ void Board::reverseSlide(int x, int y, int slideX1, int slideY1, int slideX2, in
 
 void Board::performSlide(int x, int y, Path &slide)
 {
+    kDebug() << "Entry";
     //kDebug() << "performSlide" << x << " " << y;
 
     // check if there is something to slide
@@ -828,6 +858,7 @@ void Board::performSlide(int x, int y, Path &slide)
 
 void Board::performMove(PossibleMove &possibleMoves)
 {
+    kDebug() << "Entry";
     m_connection = possibleMoves.m_path;
 #ifdef DEBUGGING
     // DEBUG undo, save board state
@@ -908,10 +939,18 @@ void Board::performMove(PossibleMove &possibleMoves)
 
 void Board::marked(int x, int y)
 {
+    kDebug() << "Entry";
+    if (field(x, y) == EMPTY) { // click on empty space on the board
+        return;
+    }
     // make sure that the previous connection is correctly undrawn
-    undrawConnection();
+    undrawConnection(); // is this still needed? (schwarzer)
 
-    if (x == m_markX && y == m_markY) {
+    if (Prefs::sounds()) {
+        playSound(KStandardDirs::locate("sound", "kshisen/tile-touch.ogg"));
+    }
+
+    if (x == m_markX && y == m_markY) { // the piece is already marked
         // unmark the piece
         m_markX = -1;
         m_markY = -1;
@@ -922,10 +961,7 @@ void Board::marked(int x, int y)
         return;
     }
 
-    if (m_markX == -1) {
-        if (field(x, y) == EMPTY) {
-            return;
-        }
+    if (m_markX == -1) { // nothing is selected so far
         m_markX = x;
         m_markY = y;
         drawPossibleMoves(false);
@@ -944,9 +980,6 @@ void Board::marked(int x, int y)
             }
         }
     }
-    if (field(x, y) == EMPTY) {
-        return;
-    }
 
     int field1 = field(m_markX, m_markY);
     int field2 = field(x, y);
@@ -991,6 +1024,7 @@ void Board::marked(int x, int y)
 
 void Board::clearHighlight()
 {
+    kDebug() << "Entry";
     if (m_highlightedTile != -1) {
         int oldHighlighted = m_highlightedTile;
         m_highlightedTile = -1;
@@ -1005,14 +1039,9 @@ void Board::clearHighlight()
     }
 }
 
-/**
- * @param x1 x coordinate of the first tile
- * @param y1 y coordinate of the first tile
- * @param x2 x coordinate of the second tile
- * @param y2 y coordinate of the second tile
- */
 bool Board::canMakePath(int x1, int y1, int x2, int y2) const
 {
+    kDebug() << "Entry";
     if (x1 == x2) {
         for (int i = qMin(y1, y2) + 1; i < qMax(y1, y2); ++i) {
             if (field(x1, i) != EMPTY) {
@@ -1034,15 +1063,9 @@ bool Board::canMakePath(int x1, int y1, int x2, int y2) const
     return false;
 }
 
-/**
- * @param x1 x coordinate of the slide's initial position
- * @param y1 y coordinate of the slide's initial position
- * @param x2 x coordinate of the slide's final position
- * @param y2 y coordinate of the slide's final position
- * @param path The movement of the last tile slided will be stored in the path
- */
 bool Board::canSlideTiles(int x1, int y1, int x2, int y2, Path &path) const
 {
+    kDebug() << "Entry";
     int distance = -1;
     path.clear();
     if (x1 == x2) {
@@ -1211,16 +1234,9 @@ bool Board::canSlideTiles(int x1, int y1, int x2, int y2, Path &path) const
     return false;
 }
 
-/**
-* @param x1 x coordinate of the first tile
-* @param y1 y coordinate of the first tile
-* @param x2 x coordinate of the second tile
-* @param y2 y coordinate of the second tile
-* @param possibleMoves All the possible moves are stored here
-* @return The number of paths found
-*/
 int Board::findPath(int x1, int y1, int x2, int y2, PossibleMoves &possibleMoves) const
 {
+    kDebug() << "Entry";
     possibleMoves.clear();
 
     int numberOfPaths = 0;
@@ -1255,16 +1271,9 @@ int Board::findPath(int x1, int y1, int x2, int y2, PossibleMoves &possibleMoves
     return numberOfPaths;
 }
 
-/**
- * @param x1 x coordinate of the first tile
- * @param y1 y coordinate of the first tile
- * @param x2 x coordinate of the second tile
- * @param y2 y coordinate of the second tile
- * @param possibleMoves All the possible moves are stored here
- * @return The number of paths found
- */
 int Board::findSimplePath(int x1, int y1, int x2, int y2, PossibleMoves &possibleMoves) const
 {
+    kDebug() << "Entry";
     int numberOfPaths = 0;
     Path path;
     // Find direct line (path of 1 segment)
@@ -1337,6 +1346,7 @@ int Board::findSimplePath(int x1, int y1, int x2, int y2, PossibleMoves &possibl
 
 void Board::drawPossibleMoves(bool b)
 {
+    kDebug() << "Entry";
     if (m_possibleMoves.isEmpty()) {
         return;
     }
@@ -1347,6 +1357,8 @@ void Board::drawPossibleMoves(bool b)
 
 void Board::drawConnection(int timeout)
 {
+    m_paintInProgress = true;
+    kDebug() << "Entry";
     if (m_connection.isEmpty()) {
         return;
     }
@@ -1366,18 +1378,15 @@ void Board::drawConnection(int timeout)
 
 void Board::undrawConnection()
 {
+    kDebug() << "Entry";
     if (m_tileRemove1.first != -1) {
         setField(m_tileRemove1.first, m_tileRemove1.second, EMPTY);
         setField(m_tileRemove2.first, m_tileRemove2.second, EMPTY);
         m_tileRemove1.first = -1;
-#ifdef USE_UPDATE
         update();
-#else
-        repaint();
-#endif
     }
 
-    gravity(true);
+    gravity(true); // why is this called here? (schwarzer)
 
     // is already undrawn?
     if (m_connection.isEmpty()) {
@@ -1415,10 +1424,12 @@ void Board::undrawConnection()
         m_gameClock.pause();
         emit endOfGame();
     }
+    m_paintInProgress = false;
 }
 
 QPoint Board::midCoord(int x, int y) const
 {
+    kDebug() << "Entry";
     QPoint p;
     int w = m_tiles.qWidth() * 2;
     int h = m_tiles.qHeight() * 2;
@@ -1444,6 +1455,7 @@ QPoint Board::midCoord(int x, int y) const
 
 void Board::setDelay(int newValue)
 {
+    kDebug() << "Entry";
     if (m_delay == newValue) {
         return;
     }
@@ -1457,6 +1469,7 @@ int Board::delay() const
 
 void Board::madeMove(int x1, int y1, int x2, int y2)
 {
+    kDebug() << "Entry";
     Path slide;
     slide.clear();
     madeMoveWithSlide(x1, y1, x2, y2, slide);
@@ -1464,6 +1477,7 @@ void Board::madeMove(int x1, int y1, int x2, int y2)
 
 void Board::madeMoveWithSlide(int x1, int y1, int x2, int y2, Path &slide)
 {
+    kDebug() << "Entry";
     Move *move;
     if (slide.empty()) {
         move = new Move(x1, y1, x2, y2, field(x1, y1), field(x2, y2));
@@ -1735,6 +1749,7 @@ void Board::redo()
 
 void Board::showHint()
 {
+    kDebug() << "Entry";
     undrawConnection();
 
     if (hint_I(m_possibleMoves)) {
@@ -1793,11 +1808,9 @@ void Board::dumpBoard(const int *board) const
 }
 #endif
 
-/**
- * The line width should be relative to the tile size, however, if the tile size is too small, keep a minimum line width.
- */
 int Board::lineWidth() const
 {
+    kDebug() << "Entry";
     int width = qRound(m_tiles.height() / 10.0);
     if (width < 3) {
         width = 3;
@@ -1808,6 +1821,7 @@ int Board::lineWidth() const
 
 bool Board::hint_I(PossibleMoves &possibleMoves) const
 {
+    kDebug() << "Entry";
     //dumpBoard();
     short done[Board::nTiles];
     for (short i = 0; i < Board::nTiles; ++i) {
@@ -1844,6 +1858,7 @@ bool Board::hint_I(PossibleMoves &possibleMoves) const
 
 void Board::setShuffle(int newValue)
 {
+    kDebug() << "Entry";
     if (m_shuffle == newValue) {
         return;
     }
@@ -1879,6 +1894,7 @@ int Board::currentTime() const
 
 bool Board::solvable(bool noRestore)
 {
+    kDebug() << "Entry";
     int *oldField = 0;
 
     if (!noRestore) {
@@ -1913,6 +1929,7 @@ bool Board::solvableFlag() const
 
 void Board::setSolvableFlag(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_solvableFlag == enabled) {
         return;
     }
@@ -1930,6 +1947,7 @@ bool Board::gravityFlag() const
 
 void Board::setGravityFlag(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_gravityFlag == enabled) {
         return;
     }
@@ -1942,16 +1960,24 @@ void Board::setGravityFlag(bool enabled)
 
 void Board::setChineseStyleFlag(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_chineseStyleFlag == enabled) {
+        kDebug() << "m_chineseStyleFlag == enabled";
         return;
     }
     m_chineseStyleFlag = enabled;
+    if (m_chineseStyleFlag) {
+        kDebug() << "==== Chinese flag is now set.";
+    } else {
+        kDebug() << "==== Chinese flag is now unset.";
+    }
     // we need to force a newGame() because board generation is different
     newGame();
 }
 
 void Board::setTilesCanSlideFlag(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_tilesCanSlideFlag == enabled) {
         return;
     }
@@ -1963,6 +1989,7 @@ void Board::setTilesCanSlideFlag(bool enabled)
 
 void Board::setPauseEnabled(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_isPaused == enabled) {
         return;
     }
@@ -1978,6 +2005,7 @@ void Board::setPauseEnabled(bool enabled)
 
 QSize Board::sizeHint() const
 {
+    kDebug() << "Entry";
     int dpi = logicalDpiX();
     if (dpi < 75) {
         dpi = 75;
@@ -2010,6 +2038,7 @@ void Board::resetRedo()
 
 void Board::setGameStuckEnabled(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_isStuck == enabled) {
         return;
     }
@@ -2025,6 +2054,7 @@ void Board::setGameStuckEnabled(bool enabled)
 
 void Board::setGameOverEnabled(bool enabled)
 {
+    kDebug() << "Entry";
     if (m_isOver == enabled) {
         return;
     }
@@ -2042,38 +2072,46 @@ void Board::setCheatModeEnabled(bool enabled)
     emit cheatStatusChanged();
 }
 
-/**
- * @return True if game is over, False if game is not over
- */
 bool Board::isOver() const
 {
     return m_isOver;
 }
 
-/**
- * @return True if game is paused, False if game is not paused
- */
 bool Board::isPaused() const
 {
     return m_isPaused;
 }
 
-/**
- * @return True if there are no matching tiles left, False if there are matching tiles left
- */
 bool Board::isStuck() const
 {
     return m_isStuck;
 }
 
-/**
-* @return True if the player is in cheat mode, False if not
-*/
 bool Board::hasCheated() const
 {
     return m_cheat;
 }
 
+void Board::setSoundsEnabled(bool enabled)
+{
+    if (enabled) {
+        m_media = new Phonon::MediaObject(this);
+        Phonon::AudioOutput *audioOutput = new Phonon::AudioOutput(Phonon::GameCategory, this);
+        Phonon::createPath(m_media, audioOutput);
+    } else {
+        delete m_media;
+        m_media = 0;
+    }
+    Prefs::setSounds(enabled);
+    Prefs::self()->writeConfig();
+}
+
+void Board::playSound(const QString &sound)
+{
+    m_media->setCurrentSource(sound);
+    m_media->play();
+}
+
 #include "board.moc"
 
 // vim: expandtab:tabstop=4:shiftwidth=4
